--[[
    Raw Ore Blocks, a mod for Minetest
    Copyright © 2022, Silver Sandstone <650286@protonmail.com>

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
]]


local S = lumpblocks.S;


function lumpblocks.register_lump_block(options)
    --[[
        lumpblocks.register_lump_block{base_name: string, lump_name: string, <options>} -> string or nil

        Options:
            name: string        Overrides the name of the node.
            texture: string     Provides a custom texture.
            colour: string      Defines the colour of the texture.
            description: string Overrides the description of the node.

        Registers a block variant of the specified lump item. At least
        `colour` or `texture` must be specified.

        Returns the node name on success, or nil on failure.
    ]]

    local base_name;
    local lump_name;
    local name;
    local texture;
    local colour;
    local description;
    for key, value in pairs(options) do
        if key == 'base_name' or key == 1 then
            base_name = value;
        elseif key == 'lump' or key == 2 then
            lump_name = value;
        elseif key == 'name' then
            name = value;
        elseif key == 'texture' then
            texture = value;
        elseif key == 'colour' then
            colour = value;
        elseif key == 'description' then
            description = value;
        else
            minetest.log('error', ('Invalid key [%q] in lumpblocks.register_lump_block().'):format(key));
        end;
    end;

    assert(lump_name,         'No lump name specified!');
    assert(base_name,         'No base name specified!');
    assert(texture or colour, 'No texture or colour specified!');

    local lump_def = minetest.registered_items[lump_name];
    if not lump_def then
        return nil;
    end;

    name = name or ('lumpblocks:%s_block'):format(base_name);

    minetest.register_node(name,
    {
        description = description or S('@1 Block', lump_def.description);
        groups      = {cracky = 3, lump_block = 1};
        tiles       = {texture or lumpblocks.generate_lumpblock_texture(options.colour)};
    });

    minetest.register_craft(
    {
        output = name;
        recipe =
        {
            {lump_name, lump_name, lump_name},
            {lump_name, lump_name, lump_name},
            {lump_name, lump_name, lump_name},
        };
    });

    minetest.register_craft(
    {
        type   = 'shapeless';
        output = lump_name .. ' 9';
        recipe = {name};
    });

    return name;
end;


function lumpblocks.generate_lumpblock_texture(colour, base)
    --[[
        lumpblocks.generate_lumpblock_texture(colour: ColourSpec, base: string or nil) -> string

        Generates a raw ore block texture in the specified colour.
    ]]

    inverted = lumpblocks.invert_colour(colour);
    if not base then
        base = 'lumpblocks_lump_block.png';
    end;
    return ('(((%s)^[invert:rgb)^[multiply:%s^[invert:rgb)^((%s)^[multiply:%s^[opacity:127)'):format(base, inverted, base, colour);
end;


function lumpblocks.invert_colour(colour)
    --[[
        lumpblocks.invert_colour(colour: ColourSpec) -> string

        Inverts a colour in RGB space.
    ]]

    local bytes = minetest.colorspec_to_bytes(colour);
    local values = {nil, nil, nil, bytes:byte(4)};
    for index = 1, 3 do
        values[index] = 255 - bytes:byte(index);
    end;
    return ('#%02X%02X%02X%02X'):format(unpack(values));
end
